# -*- coding: utf-8 -*-

"""
mistapi

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from mistapi.api_helper import APIHelper
from mistapi.configuration import Server
from mistapi.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from mistapi.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from apimatic_core.authentication.multiple.or_auth_group import Or
from mistapi.models.admin import Admin
from mistapi.exceptions.api_exception import APIException
from mistapi.exceptions.api_v_1_msps_invites_401_error_exception import ApiV1MspsInvites401ErrorException
from mistapi.exceptions.api_v_1_msps_invites_403_error_exception import ApiV1MspsInvites403ErrorException
from mistapi.exceptions.api_v_1_msps_invites_404_error_exception import ApiV1MspsInvites404ErrorException


class MspsInvitesController(BaseController):

    """A Controller to access Endpoints in the mistapi API."""
    def __init__(self, config):
        super(MspsInvitesController, self).__init__(config)

    def invite_msp_admin(self,
                         msp_id,
                         body=None):
        """Does a POST request to /api/v1/msps/{msp_id}/invites.

        Invite MSP Admin
        **Note**: An email will also be sent to the user with a link to
        https://manage.mist.com/verify/invite?token=:token

        Args:
            msp_id (uuid|string): TODO: type description here.
            body (Admin, optional): Request Body

        Returns:
            Admin: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/invites')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Admin.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsInvites401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsInvites403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsInvites404ErrorException)
        ).execute()

    def uninvite_msp_admin(self,
                           msp_id,
                           invite_id):
        """Does a DELETE request to /api/v1/msps/{msp_id}/invites/{invite_id}.

        Delete admin invite

        Args:
            msp_id (uuid|string): TODO: type description here.
            invite_id (uuid|string): TODO: type description here.

        Returns:
            object: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/invites/{invite_id}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('invite_id')
                            .value(invite_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsInvites401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsInvites403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsInvites404ErrorException)
        ).execute()

    def update_msp_admin_invite(self,
                                msp_id,
                                invite_id,
                                body=None):
        """Does a PUT request to /api/v1/msps/{msp_id}/invites/{invite_id}.

        Update MSP admin invite

        Args:
            msp_id (uuid|string): TODO: type description here.
            invite_id (uuid|string): TODO: type description here.
            body (Admin, optional): Request Body

        Returns:
            Admin: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/invites/{invite_id}')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('invite_id')
                            .value(invite_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Admin.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsInvites401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsInvites403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsInvites404ErrorException)
        ).execute()
