# -*- coding: utf-8 -*-

"""
mistapi

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from mistapi.api_helper import APIHelper
from mistapi.configuration import Server
from mistapi.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from mistapi.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from apimatic_core.authentication.multiple.or_auth_group import Or
from mistapi.models.ssorole import Ssorole
from mistapi.exceptions.api_exception import APIException
from mistapi.exceptions.api_v_1_msps_ssoroles_401_error_exception import ApiV1MspsSsoroles401ErrorException
from mistapi.exceptions.api_v_1_msps_ssoroles_403_error_exception import ApiV1MspsSsoroles403ErrorException
from mistapi.exceptions.api_v_1_msps_ssoroles_404_error_exception import ApiV1MspsSsoroles404ErrorException


class MspsSSORolesController(BaseController):

    """A Controller to access Endpoints in the mistapi API."""
    def __init__(self, config):
        super(MspsSSORolesController, self).__init__(config)

    def list_msp_sso_roles(self,
                           msp_id):
        """Does a GET request to /api/v1/msps/{msp_id}/ssoroles.

        Get List of MSP SSO Roles

        Args:
            msp_id (uuid|string): TODO: type description here.

        Returns:
            list of Ssorole: Response from the API. Example response

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/ssoroles')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Ssorole.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsSsoroles401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsSsoroles403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsSsoroles404ErrorException)
        ).execute()

    def create_msp_sso_role(self,
                            msp_id,
                            body=None):
        """Does a POST request to /api/v1/msps/{msp_id}/ssoroles.

        Create MSP Role

        Args:
            msp_id (uuid|string): TODO: type description here.
            body (Ssorole, optional): Request Body

        Returns:
            Ssorole: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/ssoroles')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Ssorole.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsSsoroles401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsSsoroles403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsSsoroles404ErrorException)
        ).execute()

    def delete_msp_sso_role(self,
                            msp_id,
                            ssorole_id):
        """Does a DELETE request to /api/v1/msps/{msp_id}/ssoroles/{ssorole_id}.

        Delete MSP SSO Roles

        Args:
            msp_id (uuid|string): TODO: type description here.
            ssorole_id (uuid|string): TODO: type description here.

        Returns:
            object: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/ssoroles/{ssorole_id}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('ssorole_id')
                            .value(ssorole_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsSsoroles401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsSsoroles403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsSsoroles404ErrorException)
        ).execute()

    def update_msp_sso_role(self,
                            msp_id,
                            ssorole_id,
                            body=None):
        """Does a PUT request to /api/v1/msps/{msp_id}/ssoroles/{ssorole_id}.

        Update SSO Role

        Args:
            msp_id (uuid|string): TODO: type description here.
            ssorole_id (uuid|string): TODO: type description here.
            body (Ssorole, optional): Request Body

        Returns:
            Ssorole: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/ssoroles/{ssorole_id}')
            .http_method(HttpMethodEnum.PUT)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .template_param(Parameter()
                            .key('ssorole_id')
                            .value(ssorole_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(body))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Ssorole.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsSsoroles401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsSsoroles403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsSsoroles404ErrorException)
        ).execute()
