# -*- coding: utf-8 -*-

"""
mistapi

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from mistapi.api_helper import APIHelper
from mistapi.configuration import Server
from mistapi.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from mistapi.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from apimatic_core.authentication.multiple.and_auth_group import And
from apimatic_core.authentication.multiple.or_auth_group import Or
from mistapi.models.ticket import Ticket
from mistapi.models.count import Count
from mistapi.exceptions.api_exception import APIException
from mistapi.exceptions.api_v_1_msps_tickets_401_error_exception import ApiV1MspsTickets401ErrorException
from mistapi.exceptions.api_v_1_msps_tickets_403_error_exception import ApiV1MspsTickets403ErrorException
from mistapi.exceptions.api_v_1_msps_tickets_404_error_exception import ApiV1MspsTickets404ErrorException
from mistapi.exceptions.api_v_1_msps_tickets_count_401_error_exception import ApiV1MspsTicketsCount401ErrorException
from mistapi.exceptions.api_v_1_msps_tickets_count_403_error_exception import ApiV1MspsTicketsCount403ErrorException
from mistapi.exceptions.api_v_1_msps_tickets_count_404_error_exception import ApiV1MspsTicketsCount404ErrorException


class MspsTicketsController(BaseController):

    """A Controller to access Endpoints in the mistapi API."""
    def __init__(self, config):
        super(MspsTicketsController, self).__init__(config)

    def list_msp_tickets(self,
                         msp_id,
                         start=0,
                         end=0,
                         duration='1d'):
        """Does a GET request to /api/v1/msps/{msp_id}/tickets.

        Get List of Tickets of a MSP

        Args:
            msp_id (uuid|string): TODO: type description here.
            start (int, optional): TODO: type description here. Example: 0
            end (int, optional): TODO: type description here. Example: 0
            duration (string, optional): For historical stats and/or logs
                where time range is needed, you can specify the time range in
                a few different ways:   * ?start=1430000000&end=1430864000
                specify the start / end   * ?end=1430864000&duration=1d
                specify end time and duration   * ?duration=1d specify
                duration, end will be now() in seconds

        Returns:
            list of Ticket: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/tickets')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('start')
                         .value(start))
            .query_param(Parameter()
                         .key('end')
                         .value(end))
            .query_param(Parameter()
                         .key('duration')
                         .value(duration))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Ticket.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsTickets401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsTickets403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsTickets404ErrorException)
        ).execute()

    def count_msp_tickets(self,
                          msp_id,
                          distinct='status'):
        """Does a GET request to /api/v1/msps/{msp_id}/tickets/count.

        Count tickets

        Args:
            msp_id (uuid|string): TODO: type description here.
            distinct (Distinct2Enum, optional): TODO: type description here.
                Example: status

        Returns:
            Count: Response from the API. OK

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/api/v1/msps/{msp_id}/tickets/count')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('msp_id')
                            .value(msp_id)
                            .should_encode(True))
            .query_param(Parameter()
                         .key('distinct')
                         .value(distinct))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('global'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Count.from_dictionary)
            .local_error('400', 'The API endpoint exists but its syntax/payload is incorrect, detail may be given', APIException)
            .local_error('401', 'Unauthorized', ApiV1MspsTicketsCount401ErrorException)
            .local_error('403', 'Permission Denied', ApiV1MspsTicketsCount403ErrorException)
            .local_error('404', 'Not found. The API endpoint doesn\'t exist or resource doesn\'t exist', ApiV1MspsTicketsCount404ErrorException)
        ).execute()
